#include <stdio.h>
#if defined( sgi )
#include <alloca.h>
#else
#include <stdlib.h>
#endif

#include "mpi.h"


/*
   This subprogram defines the boundary strides of the ghost point of
   the N-dimensional local task grid
*/



/*@

  RGX_Defn_Task_Bndry_Strides - Define MPI_Datatype, Task_Bndry_Strides[ Ndim ],
                                where Task_Bndry_Strides[ii] is the Boundary
                                Strides of the ghost-points region of the
                                local task grid in dimension ii.

  Unmodified Input Variables :
. elem_type - elemental data type of the task grid.
. Ndim - Dimensionality of the Grid (Integer).
. task_Grid_endpts - Integer Array, of size 2 x Ndim, of the two end-points'
                     coordinates of the LOCAL task grid.  It is implicitly
                     assumed to be declared like task_Grid_endpts[ 2 ][ Ndim ].
. stride_width - Integer Array, of size Ndim, of the stride width of the task
                 grid.  It is implicitly assumed to be declared like 
                 stride_width[ Ndim ].

  Modified Output Variables : 
. task_bndry_strides - MPI_Datatype Array, of size Ndim, represents the
                       ghost-points region of the task grid.  It is implicitly
                       assumed to be declared like task_bndry_strides[ Ndim ].
. returned value - returns MPI_SUCCESS if completed successfully
.n

.N RGX_fortran
.N RGX_minimal_set_of_variables
.N RGX_supported_datatype
.N RGX_yes_included

@*/
int RGX_Defn_Task_Bndry_Strides(
                                 MPI_Datatype   elem_type,
                                 int            Ndim, 
                                 int            task_Grid_endpts[],
                                 int            stride_width[], 

                                 MPI_Datatype   task_bndry_strides[]
                               )
{
   /*  Local Variables  */
   int *bndry_lgths;
   int *task_grid_sz;
   int  hyper_vol;        /*  tmp. hypercube volume in N-dim  */
   int  N_blk, blk_lgth, blk_head_septn;
   int  lo_ierr;
   int  idir, ii;



/*  Allocate Memory for local variables  */

   task_grid_sz = ( int* ) alloca( Ndim * sizeof( int ) );
   bndry_lgths  = ( int* ) alloca( Ndim * sizeof( int ) );

   hyper_vol = 1;
   for ( idir = 0; idir < Ndim; idir++ )
   {
      task_grid_sz[ idir ] = task_Grid_endpts[ idir + Ndim ]
                           - task_Grid_endpts[ idir ] + 1;
      bndry_lgths[ idir ]  = task_grid_sz[ idir ]
                           + 2 * stride_width[ idir ];
      hyper_vol           *= bndry_lgths[ idir ];
   }              /*  endif of { for ( idir = 0; idir < Ndim; idir++ ) }  */

   N_blk = hyper_vol;
   blk_head_septn = 1;
   for ( idir = 0; idir < Ndim; idir++ )
   {
      N_blk          /= bndry_lgths[ idir ];
      blk_lgth        = blk_head_septn * stride_width[ idir ];
      blk_head_septn *= bndry_lgths[ idir ];
      lo_ierr = MPI_Type_vector( N_blk, blk_lgth, blk_head_septn,
                                 elem_type, &task_bndry_strides[ idir ] );
      if ( lo_ierr !=  MPI_SUCCESS )
      {
         fprintf( stderr, __FILE__": MPI_Type_vector() fails with ierr = %d\n",
                  lo_ierr );
         return lo_ierr;
      }
      lo_ierr = MPI_Type_commit( &task_bndry_strides[ idir ] );
      if ( lo_ierr !=  MPI_SUCCESS )
      {
         fprintf( stderr, __FILE__": MPI_Type_commit() fails with ierr = %d\n",
                  lo_ierr );
         return lo_ierr;
      }
   }

   return MPI_SUCCESS;

}



#ifdef FortranNoUnderscore
#  define rgx_defn_task_bndry_strides__  rgx_defn_task_bndry_strides
#endif

#ifdef FortranSingleUnderscore
#  define rgx_defn_task_bndry_strides__  rgx_defn_task_bndry_strides_
#endif

/*  Fortran interface for RGX_Defn_Task_Bndry_Strides()  */
void rgx_defn_task_bndry_strides__(
                                    MPI_Datatype *elem_type,
                                    int          *Ndim,
                                    int          *task_Grid_endpts,
                                    int          *stride_width,

                                    MPI_Datatype *task_bndry_strides,
                                    int          *io_err
                                  )
{
   *io_err = RGX_Defn_Task_Bndry_Strides( *elem_type,
                                          *Ndim, task_Grid_endpts, stride_width,
                                          task_bndry_strides );
}
