/*
   This subprogram maps the N-dim coordinates, Nplet, to 
   linear address based on the fortran's standard of memory arrangement 
   of N-dim array.  Here the Nplet(ii) starts at 0. The linear address,
   adr, is also assumed to start at 0.  So the formula of conversion is

   adr = Nplet( 0 ) * 1
       + Nplet( 1 ) * bdy_lgths(0)
       + Nplet( 2 ) * bdy_lgths(0) * bdy_lgths(1)
       + Nplet( 3 ) * bdy_lgths(0) * bdy_lgths(1) * bdy_lgths(2)
       + ..................
       + Nplet(N-1) * bdy_lgths(0) * ...... * bdy_lgths(N-2)

*/


/*@

   adr_Nplet_2_lnr - maps the N-dim coordinates, Nplet[], to
   linear address based on the fortran's convention of memory arrangement
   of N-dimensional array.  Here the Nplet(ii) starts at 0.  The linear 
   address, adr, is also assumed to start at 0.  So the formula of 
   conversion is

.vb
   adr = Nplet( 0 ) * 1
       + Nplet( 1 ) * bdy_lgths(0)
       + Nplet( 2 ) * bdy_lgths(0) * bdy_lgths(1)
       + Nplet( 3 ) * bdy_lgths(0) * bdy_lgths(1) * bdy_lgths(2)
       + ..................
       + Nplet(N-1) * bdy_lgths(0) * ...... * bdy_lgths(N-2)
.ve


  Unmodified Input Variables :
. Ndim - Dimensionality of the coordinates, i.e. Number of the elements
         in the coordinates, Nplet[].
. bdy_lgths - Integer Array, of size Ndim, holds maximum size in each
              dimension of Nplet[].
. Nplet - Integer Array, of size Ndim, the N-dim coordinates to be converted.

  Modified Output Variables :
. lnr_adr_ptr - Pointer to the linear address, adr, to be calculated with 
                the above formula.
.n

.N RGX_yes_included

@*/

void adr_Nplet_2_lnr(
                      int  Ndim,
                      int  bdy_lgths[],
                      int  Nplet[],

                      int *lnr_adr_ptr
                    )
{
   /*  Local variables: */
   int lnr_adr;
   int hyper_vol;
   int ii;


   lnr_adr = Nplet[ 0 ];
   hyper_vol = bdy_lgths[ 0 ];
   for ( ii = 1; ii < Ndim; ii++ )
   {
      lnr_adr += Nplet[ ii ] * hyper_vol;
      hyper_vol *= bdy_lgths[ ii ];
   }

   *lnr_adr_ptr = lnr_adr;
   return;
}



#ifdef FortranNoUnderscore
#   define adr_nplet_2_lnr__  adr_nplet_2_lnr
#endif

#ifdef FortranSingleUnderscore
#   define adr_nplet_2_lnr__  adr_nplet_2_lnr_
#endif

/*
    Fortran interface for adr_Nplet_2_lnr()
*/
void adr_nplet_2_lnr__(
                        int *Ndim,
                        int *bdy_lgths,
                        int *Nplet,

                        int *lnr_adr
                      )
{
   adr_Nplet_2_lnr( *Ndim, bdy_lgths, Nplet, lnr_adr );
}
