/*
   This subprogram maps the linear address, lnr_adr, to N-dim coordinates,
   Nplet, to based on the fortran's standard of memory arrangement 
   of N-dim array.  The linear address, lnr_adr, is also assumed to start
   at 0.  Here the Nplet(ii) returned starts at 0.
   So the formula of conversion is

   adr = Nplet( 0 ) * 1
       + Nplet( 1 ) * bdy_lgths(0)
       + Nplet( 2 ) * bdy_lgths(0) * bdy_lgths(1)
       + Nplet( 3 ) * bdy_lgths(0) * bdy_lgths(1) * bdy_lgths(2)
       + ..................
       + Nplet(N-1) * bdy_lgths(0) * ...... * bdy_lgths(N-2)

*/



/*@
   adr_lnr_2_Nplet - maps the linear address, lnr_adr, to N-dim coordinates,
   Nplet, to based on the fortran's standard of memory arrangement
   of N-dim array.  The linear address, lnr_adr, is also assumed to start
   at 0.  Here the Nplet(ii) returned starts at 0.
   So the formula of conversion is

.vb
   adr = Nplet( 0 ) * 1
       + Nplet( 1 ) * bdy_lgths(0)
       + Nplet( 2 ) * bdy_lgths(0) * bdy_lgths(1)
       + Nplet( 3 ) * bdy_lgths(0) * bdy_lgths(1) * bdy_lgths(2)
       + ..................
       + Nplet(N-1) * bdy_lgths(0) * ...... * bdy_lgths(N-2)
.ve


  Unmodified Input Variables :
. lnr_adr - linear address, adr, relates to N-dim coordiantes, Nplet[],
            with the above formula.
. Ndim - Dimensionality of the coordinates, i.e. Number of the elements
         in the coordinates, Nplet[].
. bdy_lgths - Integer Array, of size Ndim, holds maximum size in each
              dimension of Nplet[].

  Modified Output Variables :
. Nplet - Integer Array, of size Ndim, the N-dim coordinates to be converted
          from the given linear address, lnr_adr.
.n

.N RGX_yes_included

@*/

void adr_lnr_2_Nplet(
                      int lnr_adr,
                      int Ndim,
                      int bdy_lgths[],

                      int Nplet[]
                    )
{
   /*  Local variables: */
   int remainder;
   int hyper_vol;
   int ii;


   hyper_vol = 1;
   for ( ii = 0; ii < Ndim-1; ii++ )
      hyper_vol *= bdy_lgths[ ii ];

   remainder = lnr_adr;
   for ( ii = Ndim-1; ii > 0; ii-- )
   {
      Nplet[ ii ] = remainder / hyper_vol;
      remainder  -= Nplet[ ii ] * hyper_vol;
      hyper_vol  /= bdy_lgths[ ii - 1 ];
   }
   Nplet[ 0 ] = remainder;

   return;
}



#ifdef FortranNoUnderscore
#   define adr_lnr_2_nplet__  adr_lnr_2_nplet
#endif

#ifdef FortranSingleUnderscore
#   define adr_lnr_2_nplet__  adr_lnr_2_nplet_
#endif

/*
    Fortran interface for adr_lnr_2_Nplet()
*/
void adr_lnr_2_nplet__(
                        int *lnr_adr,
                        int *Ndim,
                        int *bdy_lgths,

                        int *Nplet
                      )
{
   adr_lnr_2_Nplet( *lnr_adr, *Ndim, bdy_lgths, Nplet );
}
